﻿using System;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;

namespace VsxToolset
{
  // ==================================================================================
  /// <summary>
  /// This enumeration declares the types of log entries that can be used for activity
  /// log. We use this types instead of __ACTIVITYLOG_ENTRYTYPE constants.
  /// </summary>
  // ==================================================================================
  public enum ActivityLogType
  {
    Information,
    Warning,
    Error
  }

  // ==================================================================================
  /// <summary>
  /// This class defines an activity log entry with its all possible user defined
  /// properties.
  /// </summary>
  // ==================================================================================
  public sealed class ActivityLogEntry
  {
    #region Private fields

    private ActivityLogType _Type = ActivityLogType.Information;
    private string _Source = "<source not declared>";
    private string _Message = "<no message specified>";
    private Guid? _Guid = null;
    private int? _Hr = null;
    private string _Path = null;

    #endregion

    #region Lifecyle methods

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an empty log entry using default property values.
    /// </summary>
    // --------------------------------------------------------------------------------
    public ActivityLogEntry()
    {
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates a log entry with the specified message.
    /// </summary>
    /// <param name="message">Message string</param>
    // --------------------------------------------------------------------------------
    public ActivityLogEntry(string message)
    {
      _Message = message;
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates a log entry with the specified source and message.
    /// </summary>
    /// <param name="source">Source string</param>
    /// <param name="message">Message string</param>
    // --------------------------------------------------------------------------------
    public ActivityLogEntry(string source, string message)
    {
      _Source = source;
      _Message = message;
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates a log entry with the specified type and message.
    /// </summary>
    /// <param name="type">Type of the log entry</param>
    /// <param name="message">Message string</param>
    // --------------------------------------------------------------------------------
    public ActivityLogEntry(ActivityLogType type, string message)
    {
      _Type = type;
      _Message = message;
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates a log entry with the specified type, source and message.
    /// </summary>
    /// <param name="type">Type of the log entry</param>
    /// <param name="source">Source string</param>
    /// <param name="message">Message string</param>
    // --------------------------------------------------------------------------------
    public ActivityLogEntry(ActivityLogType type, string source, string message)
    {
      _Type = type;
      _Source = source;
      _Message = message;
    }

    #endregion

    #region Public properties

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Gets or sets the type of the log entry.
    /// </summary>
    // --------------------------------------------------------------------------------
    public ActivityLogType Type
    {
      get { return _Type; }
      set { _Type = value; }
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Gets or sets the source of the log entry.
    /// </summary>
    // --------------------------------------------------------------------------------
    public string Source
    {
      get { return _Source; }
      set { _Source = value; }
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Gets or sets the message of the log entry.
    /// </summary>
    // --------------------------------------------------------------------------------
    public string Message
    {
      get { return _Message; }
      set { _Message = value; }
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Gets or sets the GUID of the log entry.
    /// </summary>
    // --------------------------------------------------------------------------------
    public Guid? Guid
    {
      get { return _Guid; }
      set { _Guid = value; }
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Gets or sets the HRESULT of the log entry.
    /// </summary>
    // --------------------------------------------------------------------------------
    public int? Hr
    {
      get { return _Hr; }
      set { _Hr = value; }
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Gets or sets the path of the log entry.
    /// </summary>
    // --------------------------------------------------------------------------------
    public string Path
    {
      get { return _Path; }
      set { _Path = value; }
    }

    #endregion
  }

  // ==================================================================================
  /// <summary>
  /// This static class provides methods to create VS activity log entries.
  /// </summary>
  // ==================================================================================
  public static class ActivityLog
  {
    #region Public log methods

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity logentry by the specified ActivityLogEntry instance.
    /// </summary>
    /// <param name="entry">Instance describing the log enrty properties</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogEntry entry)
    {
      if (entry.Guid == null)
      {
        if (entry.Hr == null)
        {
          if (entry.Path == null)
            LogEntry(entry.Type, entry.Source, entry.Message);
          else
            LogEntryPath(entry.Type, entry.Source, entry.Message, entry.Path);
        }
        else
        {
          if (entry.Path == null)
            LogEntryHr(entry.Type, entry.Source, entry.Message, entry.Hr.Value);
          else
            LogEntryHrPath(entry.Type, entry.Source, entry.Message, entry.Hr.Value, entry.Path);
        }
      }
      else
      {
        if (entry.Hr == null)
        {
          if (entry.Path == null)
            LogEntryGuid(entry.Type, entry.Source, entry.Message, entry.Guid.Value);
          else
            LogEntryGuidPath(entry.Type, entry.Source, entry.Message, entry.Guid.Value, entry.Path);
        }
        else
        {
          if (entry.Path == null)
            LogEntryGuidHr(entry.Type, entry.Source, entry.Message, entry.Guid.Value, entry.Hr.Value);
          else
            LogEntryGuidHrPath(entry.Type, entry.Source, entry.Message, entry.Guid.Value, entry.Hr.Value, entry.Path);
        }
      }
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified source and message.
    /// </summary>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    // --------------------------------------------------------------------------------
    public static void Write(string source, string message)
    {
      Write(ActivityLogType.Information, source, message);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified source, message and GUID.
    /// </summary>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="guid">GUID of the entry</param>
    // --------------------------------------------------------------------------------
    public static void Write(string source, string message, Guid guid)
    {
      Write(ActivityLogType.Information, source, message, guid);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified source, message, GUID 
    /// and HRESULT.
    /// </summary>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="guid">GUID of the entry</param>
    /// <param name="hr">HRESULT of the entry</param>
    // --------------------------------------------------------------------------------
    public static void Write(string source, string message, Guid guid, int hr)
    {
      Write(ActivityLogType.Information, source, message, guid, hr);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified source, message and HRESULT.
    /// </summary>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="hr">HRESULT of the entry</param>
    // --------------------------------------------------------------------------------
    public static void Write(string source, string message, int hr)
    {
      Write(ActivityLogType.Information, source, message, hr);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified source, message, GUID, 
    /// HRESULT and path.
    /// </summary>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="guid">GUID of the entry</param>
    /// <param name="hr">HRESULT of the entry</param>
    /// <param name="path">Path of the message</param>
    // --------------------------------------------------------------------------------
    public static void Write(string source, string message, Guid guid, int hr, string path)
    {
      Write(ActivityLogType.Information, source, message, guid, hr, path);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified source, message, GUID
    /// and path.
    /// </summary>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="guid">GUID of the entry</param>
    /// <param name="path">Path of the message</param>
    // --------------------------------------------------------------------------------
    public static void Write(string source, string message, Guid guid, string path)
    {
      Write(ActivityLogType.Information, source, message, guid, path);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified source, message, HRESULT 
    /// and path.
    /// </summary>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="hr">HRESULT of the entry</param>
    /// <param name="path">Path of the message</param>
    // --------------------------------------------------------------------------------
    public static void Write(string source, string message, int hr, string path)
    {
      Write(ActivityLogType.Information, source, message, hr, path);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified source, message and path.
    /// </summary>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="path">Path of the message</param>
    // --------------------------------------------------------------------------------
    public static void Write(string source, string message, string path)
    {
      Write(ActivityLogType.Information, source, message, path);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified type, source and message.
    /// </summary>
    /// <param name="type">Entry type</param>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogType type, string source, string message)
    {
      LogEntry(type, source, message);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified type, source, message 
    /// and GUID.
    /// </summary>
    /// <param name="type">Entry type</param>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="guid">GUID of the entry</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogType type, string source, string message, Guid guid)
    {
      LogEntryGuid(type, source, message, guid);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified type, source, message, GUID 
    /// and HRESULT.
    /// </summary>
    /// <param name="type">Entry type</param>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="guid">GUID of the entry</param>
    /// <param name="hr">HRESULT of the entry</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogType type, string source, string message, Guid guid, int hr)
    {
      LogEntryGuidHr(type, source, message, guid, hr);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified type, source, message 
    /// and HRESULT.
    /// </summary>
    /// <param name="type">Entry type</param>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="hr">HRESULT of the entry</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogType type, string source, string message, int hr)
    {
      LogEntryHr(type, source, message, hr);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified type, source, message, GUID, 
    /// HRESULT and path.
    /// </summary>
    /// <param name="type">Entry type</param>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="guid">GUID of the entry</param>
    /// <param name="hr">HRESULT of the entry</param>
    /// <param name="path">Path of the message</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogType type, string source, string message, Guid guid, int hr, string path)
    {
      LogEntryGuidHrPath(type, source, message, guid, hr, path);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified type, source, message, GUID
    /// and path.
    /// </summary>
    /// <param name="type">Entry type</param>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="guid">GUID of the entry</param>
    /// <param name="path">Path of the message</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogType type, string source, string message, Guid guid, string path)
    {
      LogEntryGuidPath(type, source, message, guid, path);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified type, source, message, HRESULT 
    /// and path.
    /// </summary>
    /// <param name="type">Entry type</param>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="hr">HRESULT of the entry</param>
    /// <param name="path">Path of the message</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogType type, string source, string message, int hr, string path)
    {
      LogEntryHrPath(type, source, message, hr, path);
    }

    // --------------------------------------------------------------------------------
    /// <summary>
    /// Creates an activity log entry with the specified type, source, message 
    /// and path.
    /// </summary>
    /// <param name="type">Entry type</param>
    /// <param name="source">Event source</param>
    /// <param name="message">Event message</param>
    /// <param name="path">Path of the message</param>
    // --------------------------------------------------------------------------------
    public static void Write(ActivityLogType type, string source, string message, string path)
    {
      LogEntryPath(type, source, message, path);
    }

    #endregion

    #region Private log methods

    // All log methods here call the corresponding IVsActivityLog method by their arguments.
    // The Log property obtains the reference to the IVsActivityLog interface.
    // The ActivityLogType values are mapped to __ACTIVITYLOG_ENTRYTYPE values.

    private static UInt32 MapLogTypeToAle(ActivityLogType logType)
    {
      switch (logType)
      {
        case ActivityLogType.Information:
          return (UInt32)__ACTIVITYLOG_ENTRYTYPE.ALE_INFORMATION;
        case ActivityLogType.Warning:
          return (UInt32)__ACTIVITYLOG_ENTRYTYPE.ALE_WARNING;
        default:
          return (UInt32)__ACTIVITYLOG_ENTRYTYPE.ALE_ERROR;
      }
    }

    private static IVsActivityLog Log
    {
      get { return Package.GetGlobalService(typeof (SVsActivityLog)) as IVsActivityLog; }
    }

    private static void LogEntry(ActivityLogType type, string source, string message)
    {
      IVsActivityLog log = Log;
      if (log  != null)
      {
        log.LogEntry(MapLogTypeToAle(type), source, message);
      }
    }

    private static void LogEntryGuid(ActivityLogType type, string source, string message,
      Guid guid)
    {
      IVsActivityLog log = Log;
      if (log != null)
      {
        log.LogEntryGuid(MapLogTypeToAle(type), source, message, guid);
      }
    }

    private static void LogEntryGuidHr(ActivityLogType type, string source, string message,
      Guid guid, int hr)
    {
      IVsActivityLog log = Log;
      if (log != null)
      {
        log.LogEntryGuidHr(MapLogTypeToAle(type), source, message, guid, hr);
      }
    }

    private static void LogEntryGuidHrPath(ActivityLogType type, string source, string message,
      Guid guid, int hr, string path)
    {
      IVsActivityLog log = Log;
      if (log != null)
      {
        log.LogEntryGuidHrPath(MapLogTypeToAle(type), source, message, guid, hr, path);
      }
    }

    private static void LogEntryGuidPath(ActivityLogType type, string source, string message,
      Guid guid, string path)
    {
      IVsActivityLog log = Log;
      if (log != null)
      {
        log.LogEntryGuidPath(MapLogTypeToAle(type), source, message, guid, path);
      }
    }

    private static void LogEntryHr(ActivityLogType type, string source, string message,
      int hr)
    {
      IVsActivityLog log = Log;
      if (log != null)
      {
        log.LogEntryHr(MapLogTypeToAle(type), source, message, hr);
      }
    }

    private static void LogEntryHrPath(ActivityLogType type, string source, string message,
      int hr, string path)
    {
      IVsActivityLog log = Log;
      if (log != null)
      {
        log.LogEntryHrPath(MapLogTypeToAle(type), source, message, hr, path);
      }
    }

    private static void LogEntryPath(ActivityLogType type, string source, string message,
      string path)
    {
      IVsActivityLog log = Log;
      if (log != null)
      {
        log.LogEntryPath(MapLogTypeToAle(type), source, message, path);
      }
    }

    #endregion
  }
}
